#!/usr/bin/env ruby

# This example displays all possible combinations of font families &&
# styles.  This example also shows how to create a simple Tkinter
# based GUI for VTK-Ruby.

require 'vtk'
require 'vtk/tk'

# We set the font size constraints, default text && colors
current_font_size = 16
min_font_size = 3
max_font_size = 50

default_text = "ABCDEFGHIJKLMnopqrstuvwxyz(0123456789, !@#%-=_+.:,./<>?"

# set default_text "The quick red fox"

text_color = [246/255.0, 255/255.0, 11/255.0]
bg_color = [56/255.0, 56/255.0, 154/255.0]

# We create the render window which will show up on the screen
# We put our renderer into the render window using AddRenderer. 
# Do  !set the size of the window here.
renWin = Vtk::RenderWindow.new
ren = Vtk::Renderer.new
ren.SetBackground(bg_color)
renWin.AddRenderer(ren)

# We create text actors for each font family && several combinations
# of bold, italic && shadowed style.
text_actors = []
for family in ["Arial", "Courier", "Times"]
    for colors in [[0, 0, 0], [0, 0, 1], [1, 0, 0], [0, 1, 0], [1, 1, 0]]
        bold, italic, shadow = colors
        mapper = Vtk::TextMapper.new
        attribs = []
        if bold
            attribs.push("b")
        end
        if italic
            attribs.push("i")
        end
        if shadow
            attribs.push("s")
        end

        face_name = family
        if attribs.length
            face_name = face_name + "(" + attribs.join(",") + ")"
        end

        mapper.SetInput(face_name + ": " + default_text)        
        tprop = mapper.GetTextProperty
        eval("tprop.SetFontFamilyTo%s"%family)
        tprop.SetColor(text_color)
        tprop.SetBold(bold)
        tprop.SetItalic(italic)
        tprop.SetShadow(shadow)

        actor = Vtk::Actor2D.new
        actor.SetMapper(mapper)
        text_actors.push(actor)
        ren.AddActor(actor)
    end
end


# Now setup the Tk GUI.

# Create the root window.
root = TkRoot.new

# vtkTkRenderWindowInteractor is a Tk widget that we can render into.
# It has a GetRenderWindow method that returns a vtkRenderWindow.
# This can then be used to create a vtkRenderer && etc. We can also
# specify a vtkRenderWindow to be used when creating the widget by
# using the rw keyword argument, which is what we do here by using
# renWin. It also takes width && height options that can be used to
# specify the widget size, hence the render window size.
vtkw = Vtk::TkRenderWindowInteractor.new(root, 'rw'=>renWin, 'width'=>800)


# Once the VTK widget has been created it can be inserted into a whole
# Tk GUI as well as any other standard Tk widgets.

# This function is called by the slider-widget handler whenever the
# slider value changes (either through user interaction ||
# programmatically). It receives the slider value as parameter. We
# update the corresponding VTK objects by calling the SetFontSize
# method using this parameter && we render the scene to update the
# pipeline.
set_font_size = Proc.new{|sz|
  size = sz.to_i
  i = 0
  for actor in text_actors
    i += 1
    actor.GetMapper.GetTextProperty.SetFontSize(size)
    actor.SetDisplayPosition(10, i*(size+5))
  end

  renWin.SetSize(800, 20+i*(size+5))
  renWin.Render
}

# We create a size slider controlling the font size.  The orientation
# of this widget is horizontal (orient option). We label it using the
# label option. Finally, we bind the scale to Ruby code by assigning
# the command option to the name of a Ruby function.  Whenever the
# slider value changes this function will be called, enabling us to
# propagate this GUI setting to the corresponding VTK object.
size_slider = TkScale.new(root,
                          'from'=>min_font_size, 'to'=>max_font_size,
                          'res'=>1, 'orient'=>'horizontal',
                          'label'=>"Font size:",
                          'command'=>set_font_size)
size_slider.set(current_font_size)

# Finally we pack the VTK widget && the sliders on top of each other 
# (side='top') inside the main root widget.
vtkw.Initialize
size_slider.pack('side'=>"top", 'fill'=>"both")
vtkw.pack('side'=>"top", 'fill'=>'both', 'expand'=>1)


# Define a quit method that exits cleanly.
quit = Proc.new{
  exit
}

# We handle the WM_DELETE_WINDOW protocal request. This request is
# triggered when the widget is closed using the standard window
# manager icons || buttons. In this case the quit function will be
# called && it will free up any objects we created then exit the
# application.
root.protocol("WM_DELETE_WINDOW", quit)

renWin.Render
vtkw.Start

# start the Tkinter event loop.
root.mainloop
