#!/usr/bin/env ruby

# This example demonstrates cell picking using vtkCellPicker.  It
# displays the results of picking using a vtkTextMapper.

require 'vtk'

# create a sphere source, mapper, && actor
sphere = Vtk::SphereSource.new
sphereMapper = Vtk::PolyDataMapper.new
sphereMapper.SetInputConnection(sphere.GetOutputPort)
Vtk::PolyDataMapper.GlobalImmediateModeRenderingOn
sphereActor = Vtk::LODActor.new
sphereActor.SetMapper(sphereMapper)

# create the spikes by glyphing the sphere with a cone.  Create the
# mapper && actor for the glyphs.
cone = Vtk::ConeSource.new
glyph = Vtk::Glyph3D.new
glyph.SetInputConnection(sphere.GetOutputPort)
glyph.SetSource(cone.GetOutput)
glyph.SetVectorModeToUseNormal
glyph.SetScaleModeToScaleByVector
glyph.SetScaleFactor(0.25)
spikeMapper = Vtk::PolyDataMapper.new
spikeMapper.SetInputConnection(glyph.GetOutputPort)
spikeActor = Vtk::LODActor.new
spikeActor.SetMapper(spikeMapper)

# Create a text mapper && actor to display the results of picking.
textMapper = Vtk::TextMapper.new
tprop = textMapper.GetTextProperty
tprop.SetFontFamilyToArial
tprop.SetFontSize(10)
tprop.BoldOn
tprop.ShadowOn
tprop.SetColor(1, 0, 0)
textActor = Vtk::Actor2D.new
textActor.VisibilityOff
textActor.SetMapper(textMapper)

# Create a cell picker.
picker = Vtk::CellPicker.new

# Create a Ruby function to create the text for the text mapper used
# to display the results of picking.
annotatePick = Proc.new{|object, event|
    if picker.GetCellId < 0
        textActor.VisibilityOff
    else        selPt = picker.GetSelectionPoint
        pickPos = picker.GetPickPosition
        textMapper.SetInput("(%.6f, %.6f, %.6f)"%pickPos)
        textActor.SetPosition(selPt[0..1])
        textActor.VisibilityOn
    end
}

# Now at the end of the pick event call the above function.
picker.AddObserver("EndPickEvent", annotatePick)

# Create the Renderer, RenderWindow, etc. && set the Picker.
ren = Vtk::Renderer.new
renWin = Vtk::RenderWindow.new
renWin.AddRenderer(ren)
iren = Vtk::RenderWindowInteractor.new
iren.SetRenderWindow(renWin)
iren.SetPicker(picker)

# Add the actors to the renderer, set the background && size
ren.AddActor2D(textActor)
ren.AddActor(sphereActor)
ren.AddActor(spikeActor)
ren.SetBackground(1, 1, 1)
renWin.SetSize(300, 300)

# Get the camera && zoom in closer to the image.
ren.ResetCamera
cam1 = ren.GetActiveCamera
cam1.Zoom(1.4)

iren.Initialize
# Initially pick the cell at this location.
picker.Pick(85, 126, 0, ren)
renWin.Render
iren.Start
