#!/usr/bin/env ruby

# This example demonstrates the use of vtkLabeledDataMapper.  This
# class is used for displaying numerical data from an underlying data
# set.  In the case of this example, the underlying data are the point
# && cell ids.

require 'vtk'

# Create a selection window.  We will display the point && cell ids
# that lie within this window.
xmin = 200
@xLength = 100
xmax = xmin + @xLength
ymin = 200
@yLength = 100
ymax = ymin + @yLength

@pts = Vtk::Points.new
@pts.InsertPoint(0, xmin, ymin, 0)
@pts.InsertPoint(1, xmax, ymin, 0)
@pts.InsertPoint(2, xmax, ymax, 0)
@pts.InsertPoint(3, xmin, ymax, 0)
rect = Vtk::CellArray.new
rect.InsertNextCell(5)
rect.InsertCellPoint(0)
rect.InsertCellPoint(1)
rect.InsertCellPoint(2)
rect.InsertCellPoint(3)
rect.InsertCellPoint(0)
selectRect = Vtk::PolyData.new
selectRect.SetPoints(@pts)
selectRect.SetLines(rect)
rectMapper = Vtk::PolyDataMapper2D.new
rectMapper.SetInput(selectRect)
rectActor = Vtk::Actor2D.new
rectActor.SetMapper(rectMapper)

# Create a sphere && its associated mapper && actor.
sphere = Vtk::SphereSource.new
sphereMapper = Vtk::PolyDataMapper.new
sphereMapper.SetInputConnection(sphere.GetOutputPort)
sphereMapper.GlobalImmediateModeRenderingOn
sphereActor = Vtk::Actor.new
sphereActor.SetMapper(sphereMapper)

# Generate data arrays containing point && cell ids
ids = Vtk::IdFilter.new
ids.SetInputConnection(sphere.GetOutputPort)
ids.PointIdsOn
ids.CellIdsOn
ids.FieldDataOn

# Create the renderer here because vtkSelectVisiblePoints needs it.
ren = Vtk::Renderer.new

# Create labels for points
@visPts = Vtk::SelectVisiblePoints.new
@visPts.SetInputConnection(ids.GetOutputPort)
@visPts.SetRenderer(ren)
@visPts.SelectionWindowOn
@visPts.SetSelection(xmin, xmin + @xLength, ymin, ymin + @yLength)

# Create the mapper to display the point ids.  Specify the format to
# use for the labels.  Also create the associated actor.
ldm = Vtk::LabeledDataMapper.new
ldm.SetInputConnection(@visPts.GetOutputPort)
ldm.SetLabelFormat("%g")
ldm.SetLabelModeToLabelFieldData
pointLabels = Vtk::Actor2D.new
pointLabels.SetMapper(ldm)

# Create labels for cells
cc = Vtk::CellCenters.new
cc.SetInputConnection(ids.GetOutputPort)
@visCells = Vtk::SelectVisiblePoints.new
@visCells.SetInputConnection(cc.GetOutputPort)
@visCells.SetRenderer(ren)
@visCells.SelectionWindowOn
@visCells.SetSelection(xmin, xmin + @xLength, ymin, ymin + @yLength)

# Create the mapper to display the cell ids.  Specify the format to
# use for the labels.  Also create the associated actor.
cellMapper = Vtk::LabeledDataMapper.new
cellMapper.SetInputConnection(@visCells.GetOutputPort)
cellMapper.SetLabelFormat("%g")
cellMapper.SetLabelModeToLabelFieldData
cellMapper.GetLabelTextProperty.SetColor(0, 1, 0)
cellLabels = Vtk::Actor2D.new
cellLabels.SetMapper(cellMapper)

# Create the RenderWindow && RenderWindowInteractor
@renWin = Vtk::RenderWindow.new
@renWin.AddRenderer(ren)
iren = Vtk::RenderWindowInteractor.new
iren.SetRenderWindow(@renWin)

# Add the actors to the renderer; set the background && size;
# render
ren.AddActor(sphereActor)
ren.AddActor2D(rectActor)
ren.AddActor2D(pointLabels)
ren.AddActor2D(cellLabels)

ren.SetBackground(1, 1, 1)
@renWin.SetSize(500, 500)

# Create a function to move the selection window across the data set.
def moveWindow
    for y in 4...12
        for x in 4...12
            PlaceWindow(x*25, y*25) 
        end
    end
end


# Create a function to draw the selection window at each location it
# is moved to.
def PlaceWindow(xmin, ymin)

    xmax = xmin + @xLength
    ymax = ymin + @yLength

    @visPts.SetSelection(xmin, xmax, ymin, ymax)
    @visCells.SetSelection(xmin, xmax, ymin, ymax)

    @pts.InsertPoint(0, xmin, ymin, 0)
    @pts.InsertPoint(1, xmax, ymin, 0)
    @pts.InsertPoint(2, xmax, ymax, 0)
    @pts.InsertPoint(3, xmin, ymax, 0)
    # Call Modified because InsertPoints does  !modify vtkPoints
    # (for performance reasons)
    @pts.Modified
    @renWin.Render
end


# Initialize the interactor.
iren.Initialize
@renWin.Render

# Move the selection window across the data set.
moveWindow

# Put the selection window in the center of the render window.
# This works because the xmin = ymin = 200, xLength = yLength = 100, &&
# the render window size is 500 x 500.
PlaceWindow(xmin, ymin)

# Now start normal interaction.
iren.Start
