#!/usr/bin/env ruby

# This example demonstrates the use of vtkXYPlotActor to display three
# probe lines using three different techniques.  In this example, we
# are loading data using the vtkPLOT3DReader.  We are using the
# vtkProbeFilter to extract the underlying point data along three
# probe lines.

require 'vtk'
require 'vtk/util'
VTK_DATA_ROOT = vtkGetDataRoot

# Create a PLOT3D reader && load the data.
pl3d = Vtk::PLOT3DReader.new
pl3d.SetXYZFileName(VTK_DATA_ROOT + "/Data/combxyz.bin")
pl3d.SetQFileName(VTK_DATA_ROOT + "/Data/combq.bin")
pl3d.SetScalarFunctionNumber(100)
pl3d.SetVectorFunctionNumber(202)
pl3d.Update

# Create three the line source to use for the probe lines.
line = Vtk::LineSource.new
line.SetResolution(30)

# Move the line into place && create the probe filter.  For
# vtkProbeFilter, the probe line is the input, && the underlying data
# set is the source.
transL1 = Vtk::Transform.new
transL1.Translate(3.7, 0.0, 28.37)
transL1.Scale(5, 5, 5)
transL1.RotateY(90)
tf = Vtk::TransformPolyDataFilter.new
tf.SetInputConnection(line.GetOutputPort)
tf.SetTransform(transL1)
probe = Vtk::ProbeFilter.new
probe.SetInputConnection(tf.GetOutputPort)
probe.SetSource(pl3d.GetOutput)

# Move the line again && create another probe filter.
transL2 = Vtk::Transform.new
transL2.Translate(9.2, 0.0, 31.20)
transL2.Scale(5, 5, 5)
transL2.RotateY(90)
tf2 = Vtk::TransformPolyDataFilter.new
tf2.SetInputConnection(line.GetOutputPort)
tf2.SetTransform(transL2)
probe2 = Vtk::ProbeFilter.new
probe2.SetInputConnection(tf2.GetOutputPort)
probe2.SetSource(pl3d.GetOutput)

# Move the line again && create a third probe filter.
transL3 = Vtk::Transform.new
transL3.Translate(13.27, 0.0, 33.40)
transL3.Scale(4.5, 4.5, 4.5)
transL3.RotateY(90)
tf3 = Vtk::TransformPolyDataFilter.new
tf3.SetInputConnection(line.GetOutputPort)
tf3.SetTransform(transL3)
probe3 = Vtk::ProbeFilter.new
probe3.SetInputConnection(tf3.GetOutputPort)
probe3.SetSource(pl3d.GetOutput)

# Create a vtkAppendPolyData to merge the output of the three probe
# filters into one data set.
appendF = Vtk::AppendPolyData.new
appendF.AddInput(probe.GetPolyDataOutput)
appendF.AddInput(probe2.GetPolyDataOutput)
appendF.AddInput(probe3.GetPolyDataOutput)

# Create a tube filter to represent the lines as tubes.  Set up the
# associated mapper && actor.
tuber = Vtk::TubeFilter.new
tuber.SetInputConnection(appendF.GetOutputPort)
tuber.SetRadius(0.1)
lineMapper = Vtk::PolyDataMapper.new
lineMapper.SetInputConnection(tuber.GetOutputPort)
lineActor = Vtk::Actor.new
lineActor.SetMapper(lineMapper)

# Create an xy-plot using the output of the 3 probe filters as input.
# The x-values we are plotting are arc length.
xyplot = Vtk::XYPlotActor.new
xyplot.AddInput(probe.GetOutput)
xyplot.AddInput(probe2.GetOutput)
xyplot.AddInput(probe3.GetOutput)
xyplot.GetPositionCoordinate.SetValue(0.0, 0.67, 0)
xyplot.GetPosition2Coordinate.SetValue(1.0, 0.33, 0) #relative to Position
xyplot.SetXValuesToArcLength
xyplot.SetNumberOfXLabels(6)
xyplot.SetTitle("Pressure vs. Arc Length (Zoomed View)")
xyplot.SetXTitle("")
xyplot.SetYTitle("P")
xyplot.SetXRange(0.1, 0.35)
xyplot.SetYRange(0.2, 0.4)
xyplot.GetProperty.SetColor(0, 0, 0)
xyplot.GetProperty.SetLineWidth(2)
# Set text prop color (same color for backward compat with test)
# Assign same object to all text props
tprop = xyplot.GetTitleTextProperty
tprop.SetColor(xyplot.GetProperty.GetColor)
xyplot.SetAxisTitleTextProperty(tprop)
xyplot.SetAxisLabelTextProperty(tprop)

# Create an xy-plot using the output of the 3 probe filters as input.
# The x-values we are plotting are normalized arc length.
xyplot2 = Vtk::XYPlotActor.new
xyplot2.AddInput(probe.GetOutput)
xyplot2.AddInput(probe2.GetOutput)
xyplot2.AddInput(probe3.GetOutput)
xyplot2.GetPositionCoordinate.SetValue(0.00, 0.33, 0)
xyplot2.GetPosition2Coordinate.SetValue(1.0, 0.33, 0) #relative to Position
xyplot2.SetXValuesToNormalizedArcLength
xyplot2.SetNumberOfXLabels(6)
xyplot2.SetTitle("Pressure vs. Normalized Arc Length")
xyplot2.SetXTitle("")
xyplot2.SetYTitle("P")
xyplot2.PlotPointsOn
xyplot2.PlotLinesOff
xyplot2.GetProperty.SetColor(1, 0, 0)
xyplot2.GetProperty.SetPointSize(2)
# Set text prop color (same color for backward compat with test)
# Assign same object to all text props
tprop = xyplot2.GetTitleTextProperty
tprop.SetColor(xyplot2.GetProperty.GetColor)
xyplot2.SetAxisTitleTextProperty(tprop)
xyplot2.SetAxisLabelTextProperty(tprop)

# Create an xy-plot using the output of the 3 probe filters as input.
# The x-values we are plotting are the underlying point data values.
xyplot3 = Vtk::XYPlotActor.new
xyplot3.AddInput(probe.GetOutput)
xyplot3.AddInput(probe2.GetOutput)
xyplot3.AddInput(probe3.GetOutput)
xyplot3.GetPositionCoordinate.SetValue(0.0, 0.0, 0)
xyplot3.GetPosition2Coordinate.SetValue(1.0, 0.33, 0) #relative to Position
xyplot3.SetXValuesToIndex
xyplot3.SetNumberOfXLabels(6)
xyplot3.SetTitle("Pressure vs. Point Id")
xyplot3.SetXTitle("Probe Length")
xyplot3.SetYTitle("P")
xyplot3.PlotPointsOn
xyplot3.GetProperty.SetColor(0, 0, 1)
xyplot3.GetProperty.SetPointSize(3)
# Set text prop color (same color for backward compat with test)
# Assign same object to all text props
tprop = xyplot3.GetTitleTextProperty
tprop.SetColor(xyplot3.GetProperty.GetColor)
xyplot3.SetAxisTitleTextProperty(tprop)
xyplot3.SetAxisLabelTextProperty(tprop)

# Draw an outline of the PLOT3D data set.
outline = Vtk::StructuredGridOutlineFilter.new
outline.SetInputConnection(pl3d.GetOutputPort)
outlineMapper = Vtk::PolyDataMapper.new
outlineMapper.SetInputConnection(outline.GetOutputPort)
outlineActor = Vtk::Actor.new
outlineActor.SetMapper(outlineMapper)
outlineActor.GetProperty.SetColor(0, 0, 0)

# Create the Renderers, RenderWindow, && RenderWindowInteractor.
ren = Vtk::Renderer.new
ren2 = Vtk::Renderer.new
renWin = Vtk::RenderWindow.new
renWin.AddRenderer(ren)
renWin.AddRenderer(ren2)
iren = Vtk::RenderWindowInteractor.new
iren.SetRenderWindow(renWin)

# Set the background, viewport (necessary because we want to have the
# renderers draw to different parts of the render window) of the first
# renderer.  Add the outline && line actors to the renderer.
ren.SetBackground(0.6784, 0.8471, 0.9020)
ren.SetViewport(0, 0, 0.5, 1)
ren.AddActor(outlineActor)
ren.AddActor(lineActor)

# Set the background && viewport of the second renderer.  Add the
# xy-plot actors to the renderer.  Set the size of the render window.
ren2.SetBackground(1, 1, 1)
ren2.SetViewport(0.5, 0.0, 1.0, 1.0)
ren2.AddActor2D(xyplot)
ren2.AddActor2D(xyplot2)
ren2.AddActor2D(xyplot3)
renWin.SetSize(500, 250)

# Set up the camera parameters.
cam1 = ren.GetActiveCamera
cam1.SetClippingRange(3.95297, 100)
cam1.SetFocalPoint(8.88908, 0.595038, 29.3342)
cam1.SetPosition(-12.3332, 31.7479, 41.2387)
cam1.SetViewUp(0.060772, -0.319905, 0.945498)

iren.Initialize
renWin.Render
iren.Start
