#!/usr/bin/env ruby

# This example demonstrates the use of fields && use of
# vtkProgrammableDataObjectSource. It creates fields the hard way (as
# compared to reading a vtk field file), but shows you how to
# interface to your own raw data.

require 'vtk'
require 'vtk/util'
VTK_DATA_ROOT = vtkGetDataRoot

XAxis = "INTEREST_RATE"
YAxis = "MONTHLY_PAYMENT"
ZAxis = "MONTHLY_INCOME"
Scalar = "TIME_LATE"

def getNumberFromLine(line)
    patn = Regexp.compile('([-+]{0,1}[\d.]+e?[-+\d]*)')
   ret = []
    while patn =~ line
        line = line[line.index($1)+1..-1]
        ret.push $1.to_f
    end
    return ret
end

# Parse an ASCII file && manually create a field. Then construct a 
# dataset from the field.
@dos = Vtk::ProgrammableDataObjectSource.new

# First define the function that will parse the data.
parseFile = Proc.new{

    # Use Python to read an ASCII file
    file = File.open(File.join(VTK_DATA_ROOT, "Data/financial.txt"), "r")

    line = file.readline
    numPts = getNumberFromLine(line)[0].to_i

    numLines = (numPts - 1)/8
    # Get the data object's field data && allocate
    # room for 4, fields
    fieldData = @dos.GetOutput.GetFieldData
    fieldData.AllocateArrays(4)

    # read TIME_LATE - dependent variable
    # search the file until an array called TIME_LATE is found
    while file.readline[0..8] != "TIME_LATE"
    end

    # Create the corresponding float array
    timeLate = Vtk::FloatArray.new
    timeLate.SetName("TIME_LATE")
    # Read the values
    for i in 0...numLines
        val = getNumberFromLine(file.readline)
        for j in 0...8
            timeLate.InsertNextValue(val[j])            
        end
    end
    # Add the array
    fieldData.AddArray(timeLate)

    # MONTHLY_PAYMENT - independent variable
    while file.readline[0..14] != "MONTHLY_PAYMENT"
    end

    monthlyPayment = Vtk::FloatArray.new
    monthlyPayment.SetName("MONTHLY_PAYMENT")
    for i in 0...numLines
        val = getNumberFromLine(file.readline)
        for j in 0...8
            monthlyPayment.InsertNextValue(val[j])
        end
    end

    fieldData.AddArray(monthlyPayment)

    # UNPAID_PRINCIPLE - skip
    while file.readline[0..15] != "UNPAID_PRINCIPLE"
    end
    for i in 0...numLines
        file.readline
    end

    # LOAN_AMOUNT - skip
    while file.readline[0..10] != "LOAN_AMOUNT"
    end
    for i in 0...numLines
        file.readline
    end


    # INTEREST_RATE - independent variable
    while file.readline[0..12] != "INTEREST_RATE"
    end

    interestRate = Vtk::FloatArray.new
    interestRate.SetName("INTEREST_RATE")
    for i in 0...numLines
        val = getNumberFromLine(file.readline)
        for j in 0...8
            interestRate.InsertNextValue(val[j])
        end
    end

    fieldData.AddArray(interestRate)

    # MONTHLY_INCOME - independent variable
    while file.readline[0..13] != "MONTHLY_INCOME"
    end

    monthlyIncome = Vtk::FloatArray.new
    monthlyIncome.SetName("MONTHLY_INCOME")
    for i in 0...numLines
        val = getNumberFromLine(file.readline)
        for j in 0...8
            monthlyIncome.InsertNextValue(val[j])
        end
    end

    fieldData.AddArray(monthlyIncome)
}

# Arrange to call the parsing function when the programmable data
# source is executed.
@dos.SetExecuteMethod(parseFile)

# Create the dataset.

# DataObjectToDataSetFilter can create geometry using fields from
# DataObject's FieldData
do2ds = Vtk::DataObjectToDataSetFilter.new
do2ds.SetInputConnection(@dos.GetOutputPort)
# We are generating polygonal data
do2ds.SetDataSetTypeToPolyData
do2ds.DefaultNormalizeOn
# All we need is points. Assign them.
do2ds.SetPointComponent(0, XAxis, 0)
do2ds.SetPointComponent(1, YAxis, 0)
do2ds.SetPointComponent(2, ZAxis, 0)

# RearrangeFields is used to move fields between DataObject's
# FieldData, PointData && CellData.
rf = Vtk::RearrangeFields.new
rf.SetInputConnection(do2ds.GetOutputPort)
# Add an operation to "move TIME_LATE from DataObject's FieldData to
# PointData"
rf.AddOperation("MOVE", Scalar, "DATA_OBJECT", "POINT_DATA")
# Force the filter to execute. This is need to force the pipeline
# to execute so that we can find the range of the array TIME_LATE
rf.Update
# Set max to the second (GetRange returns [min,max]) of the "range of the 
# array called scalar in the PointData of the output of rf"
max = rf.GetOutput.GetPointData.GetArray(Scalar).GetRange[1]


# Use an ArrayCalculator to normalize TIME_LATE
calc = Vtk::ArrayCalculator.new
calc.SetInputConnection(rf.GetOutputPort)
# Working on point data
calc.SetAttributeModeToUsePointData
# Map scalar to s. When setting function, we can use s to
# represent the array scalar (TIME_LATE) 
calc.AddScalarVariable("s", Scalar, 0)
# Divide scalar by max (applies division to all components of the array)
calc.SetFunction("s / %f"%max)
# The output array will be called resArray
calc.SetResultArrayName("resArray")

# Use AssignAttribute to make resArray the active scalar field
aa = Vtk::AssignAttribute.new
aa.SetInputConnection(calc.GetOutputPort)
aa.Assign("resArray", "SCALARS", "POINT_DATA")
aa.Update

# construct pipeline for original population
# GaussianSplatter -> Contour -> Mapper -> Actor
@popSplatter = Vtk::GaussianSplatter.new
@popSplatter.SetInputConnection(aa.GetOutputPort)
@popSplatter.SetSampleDimensions(50, 50, 50)
@popSplatter.SetRadius(0.05)
@popSplatter.ScalarWarpingOff

popSurface = Vtk::ContourFilter.new
popSurface.SetInputConnection(@popSplatter.GetOutputPort)
popSurface.SetValue(0, 0.01)

popMapper = Vtk::PolyDataMapper.new
popMapper.SetInputConnection(popSurface.GetOutputPort)
popMapper.ScalarVisibilityOff

popActor = Vtk::Actor.new
popActor.SetMapper(popMapper)
popActor.GetProperty.SetOpacity(0.3)
popActor.GetProperty.SetColor(0.9, 0.9, 0.9)

# This is for decoration only.
def createAxes

    # Create axes.
    @popSplatter.Update
    bounds = @popSplatter.GetOutput.GetBounds
    axes = Vtk::Axes.new
    axes.SetOrigin(bounds[0], bounds[2], bounds[4])
    axes.SetScaleFactor(@popSplatter.GetOutput.GetLength/5.0)

    axesTubes = Vtk::TubeFilter.new
    axesTubes.SetInputConnection(axes.GetOutputPort)
    axesTubes.SetRadius(axes.GetScaleFactor/25.0)
    axesTubes.SetNumberOfSides(6)

    axesMapper = Vtk::PolyDataMapper.new
    axesMapper.SetInputConnection(axesTubes.GetOutputPort)

    axesActor = Vtk::Actor.new
    axesActor.SetMapper(axesMapper)

    # Label the axes.
    xText = Vtk::VectorText.new
    xText.SetText(XAxis)

    xTextMapper = Vtk::PolyDataMapper.new
    xTextMapper.SetInputConnection(xText.GetOutputPort)

    xActor = Vtk::Follower.new
    xActor.SetMapper(xTextMapper)
    xActor.SetScale(0.02, 0.02, 0.02)
    xActor.SetPosition(0.35, -0.05, -0.05)
    xActor.GetProperty.SetColor(0, 0, 0)

    yText = Vtk::VectorText.new
    yText.SetText(YAxis)

    yTextMapper = Vtk::PolyDataMapper.new
    yTextMapper.SetInputConnection(yText.GetOutputPort)

    yActor = Vtk::Follower.new
    yActor.SetMapper(yTextMapper)
    yActor.SetScale(0.02, 0.02, 0.02)
    yActor.SetPosition(-0.05, 0.35, -0.05)
    yActor.GetProperty.SetColor(0, 0, 0)

    zText = Vtk::VectorText.new
    zText.SetText(ZAxis)

    zTextMapper = Vtk::PolyDataMapper.new
    zTextMapper.SetInputConnection(zText.GetOutputPort)

    zActor = Vtk::Follower.new
    zActor.SetMapper(zTextMapper)
    zActor.SetScale(0.02, 0.02, 0.02)
    zActor.SetPosition(-0.05, -0.05, 0.35)
    zActor.GetProperty.SetColor(0, 0, 0)
    return axesActor, xActor, yActor, zActor
end

axesActor, xActor, yActor, zActor = createAxes

# Create the render window, renderer, interactor
ren = Vtk::Renderer.new
renWin = Vtk::RenderWindow.new
renWin.AddRenderer(ren)
renWin.SetWindowName("vtk - Field Data")
renWin.SetSize(500, 500)

iren = Vtk::RenderWindowInteractor.new
iren.SetRenderWindow(renWin)

# Add the actors to the renderer, set the background && size
ren.AddActor(axesActor)
ren.AddActor(xActor)
ren.AddActor(yActor)
ren.AddActor(xActor)
ren.AddActor(popActor)
ren.SetBackground(1, 1, 1)

# Set the default camera position
camera = Vtk::Camera.new
camera.SetClippingRange(0.274, 13.72)
camera.SetFocalPoint(0.433816, 0.333131, 0.449)
camera.SetPosition(-1.96987, 1.15145, 1.49053)
camera.SetViewUp(0.378927, 0.911821, 0.158107)
ren.SetActiveCamera(camera)
# Assign the camera to the followers.
xActor.SetCamera(camera)
yActor.SetCamera(camera)
zActor.SetCamera(camera)

iren.Initialize
renWin.Render
iren.Start
