#!/usr/bin/env ruby

# This example demonstrates the use of the vtkTransformPolyDataFilter
# to reposition a 3D text string.

require 'vtk'
require 'vtk/util'

# Define a Single Cube
Scalars = Vtk::FloatArray.new
Scalars.InsertNextValue(1.0)
Scalars.InsertNextValue(0.0)
Scalars.InsertNextValue(0.0)
Scalars.InsertNextValue(1.0)
Scalars.InsertNextValue(0.0)
Scalars.InsertNextValue(0.0)
Scalars.InsertNextValue(0.0)
Scalars.InsertNextValue(0.0)

Points = Vtk::Points.new
Points.InsertNextPoint(0, 0, 0)
Points.InsertNextPoint(1, 0, 0)
Points.InsertNextPoint(1, 1, 0)
Points.InsertNextPoint(0, 1, 0)
Points.InsertNextPoint(0, 0, 1)
Points.InsertNextPoint(1, 0, 1)
Points.InsertNextPoint(1, 1, 1)
Points.InsertNextPoint(0, 1, 1)

Ids = Vtk::IdList.new
Ids.InsertNextId(0)
Ids.InsertNextId(1)
Ids.InsertNextId(2)
Ids.InsertNextId(3)
Ids.InsertNextId(4)
Ids.InsertNextId(5)
Ids.InsertNextId(6)
Ids.InsertNextId(7)

Grid = Vtk::UnstructuredGrid.new
Grid.Allocate(10, 10)
Grid.InsertNextCell(12, Ids)
Grid.SetPoints(Points)
Grid.GetPointData.SetScalars(Scalars)

# Find the triangles that lie along the 0.5 contour in this cube.
Marching = Vtk::ContourFilter.new
Marching.SetInput(Grid)
Marching.SetValue(0, 0.5)
Marching.Update

# Extract the edges of the triangles just found.
triangleEdges = Vtk::ExtractEdges.new
triangleEdges.SetInputConnection(Marching.GetOutputPort)
# Draw the edges as tubes instead of lines.  Also create the associated
# mapper && actor to display the tubes.
triangleEdgeTubes = Vtk::TubeFilter.new
triangleEdgeTubes.SetInputConnection(triangleEdges.GetOutputPort)
triangleEdgeTubes.SetRadius(0.005)
triangleEdgeTubes.SetNumberOfSides(6)
triangleEdgeTubes.UseDefaultNormalOn
triangleEdgeTubes.SetDefaultNormal(0.577, 0.577, 0.577)
triangleEdgeMapper = Vtk::PolyDataMapper.new
triangleEdgeMapper.SetInputConnection(triangleEdgeTubes.GetOutputPort)
triangleEdgeMapper.ScalarVisibilityOff
triangleEdgeActor = Vtk::Actor.new
triangleEdgeActor.SetMapper(triangleEdgeMapper)
triangleEdgeActor.GetProperty.SetDiffuseColor(Vtk::Colors::Lamp_black)
triangleEdgeActor.GetProperty.SetSpecular(0.4)
triangleEdgeActor.GetProperty.SetSpecularPower(10)

# Shrink the triangles we found earlier.  Create the associated mapper
# && actor.  Set the opacity of the shrunken triangles.
aShrinker = Vtk::ShrinkPolyData.new
aShrinker.SetShrinkFactor(1)
aShrinker.SetInputConnection(Marching.GetOutputPort)
aMapper = Vtk::PolyDataMapper.new
aMapper.ScalarVisibilityOff
aMapper.SetInputConnection(aShrinker.GetOutputPort)
Triangles = Vtk::Actor.new
Triangles.SetMapper(aMapper)
Triangles.GetProperty.SetDiffuseColor(Vtk::Colors::Banana)
Triangles.GetProperty.SetOpacity(0.6)

# Draw a cube the same size && at the same position as the one
# created previously.  Extract the edges because we only want to see
# the outline of the cube.  Pass the edges through a vtkTubeFilter so
# they are displayed as tubes rather than lines.
CubeModel = Vtk::CubeSource.new
CubeModel.SetCenter(0.5, 0.5, 0.5)
Edges = Vtk::ExtractEdges.new
Edges.SetInputConnection(CubeModel.GetOutputPort)
Tubes = Vtk::TubeFilter.new
Tubes.SetInputConnection(Edges.GetOutputPort)
Tubes.SetRadius(0.01)
Tubes.SetNumberOfSides(6)
Tubes.UseDefaultNormalOn
Tubes.SetDefaultNormal(0.577, 0.577, 0.577)
# Create the mapper && actor to display the cube edges.
TubeMapper = Vtk::PolyDataMapper.new
TubeMapper.SetInputConnection(Tubes.GetOutputPort)
CubeEdges = Vtk::Actor.new
CubeEdges.SetMapper(TubeMapper)
CubeEdges.GetProperty.SetDiffuseColor(Vtk::Colors::Khaki)
CubeEdges.GetProperty.SetSpecular(0.4)
CubeEdges.GetProperty.SetSpecularPower(10)

# Create a sphere to use as a glyph source for vtkGlyph3D.
Sphere = Vtk::SphereSource.new
Sphere.SetRadius(0.04)
Sphere.SetPhiResolution(20)
Sphere.SetThetaResolution(20)
# Remove the part of the cube with data values below 0.5.
ThresholdIn = Vtk::ThresholdPoints.new
ThresholdIn.SetInput(Grid)
ThresholdIn.ThresholdByUpper(0.5)
# Display spheres at the vertices remaining in the cube data set after
# it was passed through vtkThresholdPoints.
Vertices = Vtk::Glyph3D.new
Vertices.SetInputConnection(ThresholdIn.GetOutputPort)
Vertices.SetSource(Sphere.GetOutput)
# Create a mapper && actor to display the glyphs.
SphereMapper = Vtk::PolyDataMapper.new
SphereMapper.SetInputConnection(Vertices.GetOutputPort)
SphereMapper.ScalarVisibilityOff
CubeVertices = Vtk::Actor.new
CubeVertices.SetMapper(SphereMapper)
CubeVertices.GetProperty.SetDiffuseColor(Vtk::Colors::Tomato)
CubeVertices.GetProperty.SetDiffuseColor(Vtk::Colors::Tomato)

# Define the text for the label
caseLabel = Vtk::VectorText.new
caseLabel.SetText("Case 1")

# Set up a transform to move the label to a new position.
aLabelTransform = Vtk::Transform.new
aLabelTransform.Identity
aLabelTransform.Translate(-0.2, 0, 1.25)
aLabelTransform.Scale(0.05, 0.05, 0.05)

# Move the label to a new position.
labelTransform = Vtk::TransformPolyDataFilter.new
labelTransform.SetTransform(aLabelTransform)
labelTransform.SetInputConnection(caseLabel.GetOutputPort)

# Create a mapper && actor to display the text.
labelMapper = Vtk::PolyDataMapper.new
labelMapper.SetInputConnection(labelTransform.GetOutputPort)

labelActor = Vtk::Actor.new
labelActor.SetMapper(labelMapper)

# Define the base that the cube sits on.  Create its associated mapper
# && actor.  Set the position of the actor.
baseModel = Vtk::CubeSource.new
baseModel.SetXLength(1.5)
baseModel.SetYLength(0.01)
baseModel.SetZLength(1.5)
baseMapper = Vtk::PolyDataMapper.new
baseMapper.SetInputConnection(baseModel.GetOutputPort)
base = Vtk::Actor.new
base.SetMapper(baseMapper)
base.SetPosition(0.5, -0.09, 0.5)

# Create the Renderer, RenderWindow, && RenderWindowInteractor
ren = Vtk::Renderer.new
renWin = Vtk::RenderWindow.new
renWin.AddRenderer(ren)
renWin.SetSize(640, 480)
iren = Vtk::RenderWindowInteractor.new
iren.SetRenderWindow(renWin)

# Add the actors to the renderer
ren.AddActor(triangleEdgeActor)
ren.AddActor(base)
ren.AddActor(labelActor)
ren.AddActor(CubeEdges)
ren.AddActor(CubeVertices)
ren.AddActor(Triangles)

# Set the background color.
ren.SetBackground(Vtk::Colors::Slate_grey)

# This sets up the right values for case12 of the marching cubes
# algorithm (routine translated from vtktesting/mccases.tcl).
def case12(scalars, caselabel, iin, out)
    scalars.InsertValue(0, out)
    scalars.InsertValue(1, iin)
    scalars.InsertValue(2, out)
    scalars.InsertValue(3, iin)
    scalars.InsertValue(4, iin)
    scalars.InsertValue(5, iin)
    scalars.InsertValue(6, out)
    scalars.InsertValue(7, out)
    if iin == 1
        caselabel.SetText("Case 12 - 00111010")
    else        caselabel.SetText("Case 12 - 11000101")
    end
end

# Set the scalar values for this case of marching cubes.
case12(Scalars, caseLabel, 0, 1)

# Force the grid to update.
Grid.Modified

# Position the camera.
ren.ResetCamera
ren.GetActiveCamera.Dolly(1.2)
ren.GetActiveCamera.Azimuth(30)
ren.GetActiveCamera.Elevation(20)
ren.ResetCameraClippingRange

iren.Initialize
renWin.Render
iren.Start
