#!/usr/bin/env ruby

# This example demonstrates the conversion of point data to cell data.
# The conversion is necessary because we want to threshold data based
# on cell scalar values.

require 'vtk'
require 'vtk/util'
VTK_DATA_ROOT = vtkGetDataRoot

# Read some data with point data attributes. The data is from a
# plastic blow molding process (e.g., to make plastic bottles) &&
# consists of two logical components: a mold && a parison. The
# parison is the hot plastic that is being molded, && the mold is
# clamped around the parison to form its shape.
reader = Vtk::UnstructuredGridReader.new
reader.SetFileName(VTK_DATA_ROOT + "/Data/blow.vtk")
reader.SetScalarsName("thickness9")
reader.SetVectorsName("displacement9")

# Convert the point data to cell data. The point data is passed
# through the filter so it can be warped. The vtkThresholdFilter then
# thresholds based on cell scalar values && extracts a portion of the
# parison whose cell scalar values lie between 0.25 && 0.75.
p2c = Vtk::PointDataToCellData.new
p2c.SetInputConnection(reader.GetOutputPort)
p2c.PassPointDataOn
warp = Vtk::WarpVector.new
warp.SetInput(p2c.GetUnstructuredGridOutput)
thresh = Vtk::Threshold.new
thresh.SetInputConnection(warp.GetOutputPort)
thresh.ThresholdBetween(0.25, 0.75)
thresh.SetInputArrayToProcess(1, 0, 0, 0, "thickness9")
#thresh.SetAttributeModeToUseCellData

# This is used to extract the mold from the parison. 
connect = Vtk::ConnectivityFilter.new
connect.SetInputConnection(thresh.GetOutputPort)
connect.SetExtractionModeToSpecifiedRegions
connect.AddSpecifiedRegion(0)
connect.AddSpecifiedRegion(1)
moldMapper = Vtk::DataSetMapper.new
moldMapper.SetInputConnection(reader.GetOutputPort)
moldMapper.ScalarVisibilityOff
moldActor = Vtk::Actor.new
moldActor.SetMapper(moldMapper)
moldActor.GetProperty.SetColor(0.2, 0.2, 0.2)
moldActor.GetProperty.SetRepresentationToWireframe

# The threshold filter has been used to extract the parison.
connect2 = Vtk::ConnectivityFilter.new
connect2.SetInputConnection(thresh.GetOutputPort)
parison = Vtk::GeometryFilter.new
parison.SetInputConnection(connect2.GetOutputPort)
normals2 = Vtk::PolyDataNormals.new
normals2.SetInputConnection(parison.GetOutputPort)
normals2.SetFeatureAngle(60)
lut = Vtk::LookupTable.new
lut.SetHueRange(0.0, 0.66667)
parisonMapper = Vtk::PolyDataMapper.new
parisonMapper.SetInputConnection(normals2.GetOutputPort)
parisonMapper.SetLookupTable(lut)
parisonMapper.SetScalarRange(0.12, 1.0)
parisonActor = Vtk::Actor.new
parisonActor.SetMapper(parisonMapper)

# We generate some contour lines on the parison.
cf = Vtk::ContourFilter.new
cf.SetInputConnection(connect2.GetOutputPort)
cf.SetValue(0, 0.5)
contourMapper = Vtk::PolyDataMapper.new
contourMapper.SetInputConnection(cf.GetOutputPort)
contours = Vtk::Actor.new
contours.SetMapper(contourMapper)

# Create graphics stuff
ren = Vtk::Renderer.new
renWin = Vtk::RenderWindow.new
renWin.AddRenderer(ren)
iren = Vtk::RenderWindowInteractor.new
iren.SetRenderWindow(renWin)

# Add the actors to the renderer, set the background && size
ren.AddActor(moldActor)
ren.AddActor(parisonActor)
ren.AddActor(contours)

ren.ResetCamera
ren.GetActiveCamera.Azimuth(60)
ren.GetActiveCamera.Roll(-90)
ren.GetActiveCamera.Dolly(2)
ren.ResetCameraClippingRange

ren.SetBackground(1, 1, 1)
renWin.SetSize(750, 400)

iren.Initialize
renWin.Render
iren.Start
