#!/usr/bin/env ruby

# Demonstrate how to use the vtkBoxWidget 3D widget. This script uses
# a 3D box widget to define a "clipping box" to clip some simple
# geometry (a mace). Make sure that you hit the "i" key to activate
# the widget.

require 'vtk'

# Create a mace out of filters.
sphere = Vtk::SphereSource.new
cone = Vtk::ConeSource.new
glyph = Vtk::Glyph3D.new
glyph.SetInputConnection(sphere.GetOutputPort)
glyph.SetSource(cone.GetOutput)
glyph.SetVectorModeToUseNormal
glyph.SetScaleModeToScaleByVector
glyph.SetScaleFactor(0.25)

# The sphere && spikes are appended into a single polydata. This just
# makes things simpler to manage.
apd = Vtk::AppendPolyData.new
apd.AddInput(glyph.GetOutput)
apd.AddInput(sphere.GetOutput)
maceMapper = Vtk::PolyDataMapper.new
maceMapper.SetInputConnection(apd.GetOutputPort)
maceActor = Vtk::LODActor.new
maceActor.SetMapper(maceMapper)
maceActor.VisibilityOn

# This portion of the code clips the mace with the vtkPlanes implicit
# function.  The clipped region is colored green.
planes = Vtk::Planes.new
clipper = Vtk::ClipPolyData.new
clipper.SetInputConnection(apd.GetOutputPort)
clipper.SetClipFunction(planes)
clipper.InsideOutOn
selectMapper = Vtk::PolyDataMapper.new
selectMapper.SetInputConnection(clipper.GetOutputPort)
selectActor = Vtk::LODActor.new
selectActor.SetMapper(selectMapper)
selectActor.GetProperty.SetColor(0, 1, 0)
selectActor.VisibilityOff
selectActor.SetScale(1.01, 1.01, 1.01)

# Create the RenderWindow, Renderer && both Actors
ren = Vtk::Renderer.new
renWin = Vtk::RenderWindow.new
renWin.AddRenderer(ren)

iren = Vtk::RenderWindowInteractor.new
iren.SetRenderWindow(renWin)

# The SetInteractor method is how 3D widgets are associated with the
# render window interactor.  Internally, SetInteractor sets up a bunch
# of callbacks using the Command/Observer mechanism (AddObserver).
boxWidget = Vtk::BoxWidget.new
boxWidget.SetInteractor(iren)
boxWidget.SetPlaceFactor(1.25)

ren.AddActor(maceActor)
ren.AddActor(selectActor)

# Add the actors to the renderer, set the background && size
ren.SetBackground(0.1, 0.2, 0.4)
renWin.SetSize(300, 300)

# This callback funciton does the actual work: updates the vtkPlanes
# implicit function.  This in turn causes the pipeline to update.
SelectPolygons = Proc.new{|object, event|
    # object will be the boxWidget
    object.GetPlanes(planes)
    selectActor.VisibilityOn
}

# Place the interactor initially. The input to a 3D widget is used to
# initially position && scale the widget. The "EndInteractionEvent" is
# observed which invokes the SelectPolygons callback.
boxWidget.SetInput(glyph.GetOutput)
boxWidget.PlaceWidget
boxWidget.AddObserver("EndInteractionEvent", SelectPolygons)

iren.Initialize
renWin.Render
iren.Start
