#!/usr/bin/env ruby

# This example creates a polygonal model of a cone, && then renders
# it to the screen. It also defines an interaction style by creating a
# set of Command/Observers. (Note: it is far more efficient to create
# new styles by subclassing vtkInteractorStyle. This is just an
# illustrative example.) If you really want trackball behavior, look
# at the vtkInteractorStyleTrackballCamera class.

require 'vtk'

# We create an instance of vtkConeSource && set some of its
# properties. The instance of vtkConeSource "cone" is part of a
# visualization pipeline (it is a source process object); it produces
# data (output type is vtkPolyData) which other filters may process.
cone = Vtk::ConeSource.new
cone.SetHeight(3.0)
cone.SetRadius(1.0)
cone.SetResolution(10)

# In this example we terminate the pipeline with a mapper process
# object.  (Intermediate filters such as vtkShrinkPolyData could be
# inserted in between the source && the mapper.)  We create an
# instance of vtkPolyDataMapper to map the polygonal data into
# graphics primitives. We connect the output of the cone souece to the
# input of this mapper.
coneMapper = Vtk::PolyDataMapper.new
coneMapper.SetInputConnection(cone.GetOutputPort)

# Create an actor to represent the cone. The actor orchestrates
# rendering of the mapper's graphics primitives. An actor also refers
# to properties via a vtkProperty instance, && includes an internal
# transformation matrix. We set this actor's mapper to be coneMapper
# which we created above.
coneActor = Vtk::Actor.new
coneActor.SetMapper(coneMapper)

# Create the Renderer && assign actors to it. A renderer is like a
# viewport. It is part || all of a window on the screen && it is
# responsible for drawing the actors it has.  We also set the
# background color here.
@ren = Vtk::Renderer.new
@ren.AddActor(coneActor)
@ren.SetBackground(0.1, 0.2, 0.4)

# Finally we create the render window which will show up on the screen
# We put our renderer into the render window using AddRenderer. We
# also set the size to be 300 pixels by 300.
@renWin = Vtk::RenderWindow.new
@renWin.AddRenderer(@ren)
@renWin.SetSize(300, 300)

# Define custom interaction.
iren = Vtk::RenderWindowInteractor.new
iren.SetInteractorStyle(nil)
iren.SetRenderWindow(@renWin)

# Add the observers to watch for particular events. These invoke
# Python functions.
rotating = 0
panning = 0
zooming = 0

# Handle the mouse button events.
ButtonEvent = Proc.new{|obj, event|
    if event == "LeftButtonPressEvent"
        rotating = 1
    elsif event == "LeftButtonReleaseEvent"
        rotating = 0
    elsif event == "MiddleButtonPressEvent"
        panning = 1
    elsif event == "MiddleButtonReleaseEvent"
        panning = 0
    elsif event == "RightButtonPressEvent"
        zooming = 1
    elsif event == "RightButtonReleaseEvent"
        zooming = 0
    end
}

# General high-level logic
MouseMove = Proc.new{|obj, event|
    lastXYpos = iren.GetLastEventPosition
    lastX = lastXYpos[0]
    lastY = lastXYpos[1]

    xypos = iren.GetEventPosition
    x = xypos[0]
    y = xypos[1]

    center = @renWin.GetSize
    centerX = center[0]/2.0
    centerY = center[1]/2.0

    if rotating
        Rotate(@ren, @ren.GetActiveCamera, x, y, lastX, lastY,
               centerX, centerY)
    elsif panning
        Pan(@ren, @ren.GetActiveCamera, x, y, lastX, lastY, centerX,
            centerY)
    elsif zooming
        Dolly(@ren, @ren.GetActiveCamera, x, y, lastX, lastY,
              centerX, centerY)
    end
}


Keypress = Proc.new{|obj, event|
    key = obj.GetKeySym
    if key == "e"
        obj.InvokeEvent("DeleteAllObjects")
        sys.exit
    elsif key == "w"
        Wireframe
    elsif key =="s"
        Surface 
    end
}


# Routines that translate the events into camera motions.

# This one is associated with the left mouse button. It translates x
# && y relative motions into camera azimuth && elevation commands.
def Rotate(renderer, camera, x, y, lastX, lastY, centerX, centerY) 
    camera.Azimuth(lastX-x)
    camera.Elevation(lastY-y)
    camera.OrthogonalizeViewUp
    @renWin.Render
end


# Pan translates x-y motion into translation of the focal point &&
# position.
def Pan(renderer, camera, x, y, lastX, lastY, centerX, centerY)
    fPoint = camera.GetFocalPoint
    fPoint0 = fPoint[0]
    fPoint1 = fPoint[1]
    fPoint2 = fPoint[2]

    pPoint = camera.GetPosition
    pPoint0 = pPoint[0]
    pPoint1 = pPoint[1]
    pPoint2 = pPoint[2]

    renderer.SetWorldPoint(FPoint0, FPoint1, FPoint2, 1.0)
    renderer.WorldToDisplay
    dPoint = renderer.GetDisplayPoint
    focalDepth = dPoint[2]

    aPoint0 = centerX+(x-lastX)
    aPoint1 = centerY+(y-lastY)

    renderer.SetDisplayPoint(APoint0, APoint1, focalDepth)
    renderer.DisplayToWorld
    rPoint = renderer.GetWorldPoint
    rPoint0 = rPoint[0]
    rPoint1 = rPoint[1]
    rPoint2 = rPoint[2]
    rPoint3 = rPoint[3]

    if rPoint3 != 0.0
        rPoint0 = rPoint0/rPoint3
        rPoint1 = rPoint1/rPoint3
        rPoint2 = rPoint2/rPoint3
    end

    camera.SetFocalPoint( (fPoint0-rPoint0)/2.0 + fPoint0,
                          (fPoint1-rPoint1)/2.0 + fPoint1,
                          (fPoint2-rPoint2)/2.0 + fPoint2)
    camera.SetPosition( (fPoint0-rPoint0)/2.0 + pPoint0,
                        (fPoint1-rPoint1)/2.0 + pPoint1,
                        (fPoint2-rPoint2)/2.0 + pPoint2)
    @renWin.Render
end


# Dolly converts y-motion into a camera dolly commands.
def Dolly(renderer, camera, x, y, lastX, lastY, centerX, centerY)
    dollyFactor = pow(1.02,(0.5*(y-lastY)))
    if camera.GetParallelProjection
        parallelScale = camera.GetParallelScale*dollyFactor
        camera.SetParallelScale(parallelScale)
    else
        camera.Dolly(dollyFactor)
        renderer.ResetCameraClippingRange
    end

    @renWin.Render 
end

# Wireframe sets the representation of all actors to wireframe.
def Wireframe
    actors = @ren.GetActors
    actors.InitTraversal
    actor = actors.GetNextItem
    while actor
        actor.GetProperty.SetRepresentationToWireframe
        actor = actors.GetNextItem
    end

    @renWin.Render 
end

# Surface sets the representation of all actors to surface.
def Surface
    actors = @ren.GetActors
    actors.InitTraversal
    actor = actors.GetNextItem
    while actor
        actor.GetProperty.SetRepresentationToSurface
        actor = actors.GetNextItem
    end
    @renWin.Render
end


iren.AddObserver("LeftButtonPressEvent", ButtonEvent)
iren.AddObserver("LeftButtonReleaseEvent", ButtonEvent)
iren.AddObserver("MiddleButtonPressEvent", ButtonEvent)
iren.AddObserver("MiddleButtonReleaseEvent", ButtonEvent)
iren.AddObserver("RightButtonPressEvent", ButtonEvent)
iren.AddObserver("RightButtonReleaseEvent", ButtonEvent)
iren.AddObserver("MouseMoveEvent", MouseMove)
iren.AddObserver("KeyPressEvent", Keypress)


iren.Initialize
@renWin.Render
iren.Start
