#!/usr/bin/env ruby

# This example reads a volume dataset, extracts two isosurfaces that
# represent the skin && bone, && then displays them.

require 'vtk'
require 'vtk/util'
VTK_DATA_ROOT = vtkGetDataRoot

# Create the renderer, the render window, && the interactor. The
# renderer draws into the render window, the interactor enables mouse-
# && keyboard-based interaction with the scene.
aRenderer = Vtk::Renderer.new
renWin = Vtk::RenderWindow.new
renWin.AddRenderer(aRenderer)
iren = Vtk::RenderWindowInteractor.new
iren.SetRenderWindow(renWin)

# The following reader is used to read a series of 2D slices (images)
# that compose the volume. The slice dimensions are set, && the
# pixel spacing. The data Endianness must also be specified. The reader
# usese the FilePrefix in combination with the slice number to construct
# filenames using the format FilePrefix.%d. (In this case the FilePrefix
# is the root name of the file: quarter.)
v16 = Vtk::Volume16Reader.new
v16.SetDataDimensions(64, 64)
v16.SetDataByteOrderToLittleEndian
v16.SetFilePrefix(VTK_DATA_ROOT + "/Data/headsq/quarter")
v16.SetImageRange(1, 93)
v16.SetDataSpacing(3.2, 3.2, 1.5)

# An isosurface, || contour value of 500 is known to correspond to the
# skin of the patient. Once generated, a vtkPolyDataNormals filter is
# is used to create normals for smooth surface shading during rendering.
# The triangle stripper is used to create triangle strips from the
# isosurface these render much faster on may systems.
skinExtractor = Vtk::ContourFilter.new
skinExtractor.SetInputConnection(v16.GetOutputPort)
skinExtractor.SetValue(0, 500)
skinNormals = Vtk::PolyDataNormals.new
skinNormals.SetInputConnection(skinExtractor.GetOutputPort)
skinNormals.SetFeatureAngle(60.0)
skinStripper = Vtk::Stripper.new
skinStripper.SetInputConnection(skinNormals.GetOutputPort)
skinMapper = Vtk::PolyDataMapper.new
skinMapper.SetInputConnection(skinStripper.GetOutputPort)
skinMapper.ScalarVisibilityOff
skin = Vtk::Actor.new
skin.SetMapper(skinMapper)
skin.GetProperty.SetDiffuseColor(1, 0.49, 0.25)
skin.GetProperty.SetSpecular(0.3)
skin.GetProperty.SetSpecularPower(20)

# An isosurface, || contour value of 1150 is known to correspond to the
# skin of the patient. Once generated, a vtkPolyDataNormals filter is
# is used to create normals for smooth surface shading during rendering.
# The triangle stripper is used to create triangle strips from the
# isosurface these render much faster on may systems.
boneExtractor = Vtk::ContourFilter.new
boneExtractor.SetInputConnection(v16.GetOutputPort)
boneExtractor.SetValue(0, 1150)
boneNormals = Vtk::PolyDataNormals.new
boneNormals.SetInputConnection(boneExtractor.GetOutputPort)
boneNormals.SetFeatureAngle(60.0)
boneStripper = Vtk::Stripper.new
boneStripper.SetInputConnection(boneNormals.GetOutputPort)
boneMapper = Vtk::PolyDataMapper.new
boneMapper.SetInputConnection(boneStripper.GetOutputPort)
boneMapper.ScalarVisibilityOff
bone = Vtk::Actor.new
bone.SetMapper(boneMapper)
bone.GetProperty.SetDiffuseColor(1, 1, 0.9412)

# An outline provides context around the data.
outlineData = Vtk::OutlineFilter.new
outlineData.SetInputConnection(v16.GetOutputPort)
mapOutline = Vtk::PolyDataMapper.new
mapOutline.SetInputConnection(outlineData.GetOutputPort)
outline = Vtk::Actor.new
outline.SetMapper(mapOutline)
outline.GetProperty.SetColor(0, 0, 0)

# It is convenient to create an initial view of the data. The FocalPoint
# && Position form a vector direction. Later on (ResetCamera method)
# this vector is used to position the camera to look at the data in
# this direction.
aCamera = Vtk::Camera.new
aCamera.SetViewUp(0, 0, -1)
aCamera.SetPosition(0, 1, 0)
aCamera.SetFocalPoint(0, 0, 0)
aCamera.ComputeViewPlaneNormal

# Actors are added to the renderer. An initial camera view is created.
# The Dolly method moves the camera towards the FocalPoint,
# thereby enlarging the image.
aRenderer.AddActor(outline)
aRenderer.AddActor(skin)
aRenderer.AddActor(bone)
aRenderer.SetActiveCamera(aCamera)
aRenderer.ResetCamera
aCamera.Dolly(1.5)

# Set a background color for the renderer && set the size of the
# render window (expressed in pixels).
aRenderer.SetBackground(1, 1, 1)
renWin.SetSize(640, 480)

# Note that when camera movement occurs (as it does in the Dolly
# method), the clipping planes often need adjusting. Clipping planes
# consist of two planes: near && far along the view direction. The
# near plane clips out objects in front of the plane the far plane
# clips out objects behind the plane. This way only what is drawn
# between the planes is actually rendered.
aRenderer.ResetCameraClippingRange

# Interact with the data.
iren.Initialize
renWin.Render
iren.Start
