#!/usr/bin/env ruby

# This example demonstrates how to use 2D Delaunay triangulation.
# We create a fancy image of a 2D Delaunay triangulation. Points are
# randomly generated.

require 'vtk'
require 'vtk/util'

# Generate some random points
math = Vtk::Math
points = Vtk::Points.new
for i in 0...50
    points.InsertPoint(i, math.Random(0, 1), math.Random(0, 1), 0.0)
end

# Create a polydata with the points we just created.
profile = Vtk::PolyData.new
profile.SetPoints(points)

# Perform a 2D Delaunay triangulation on them.
delny = Vtk::Delaunay2D.new
delny.SetInput(profile)
delny.SetTolerance(0.001)
mapMesh = Vtk::PolyDataMapper.new
mapMesh.SetInputConnection(delny.GetOutputPort)
meshActor = Vtk::Actor.new
meshActor.SetMapper(mapMesh)
meshActor.GetProperty.SetColor(0.1, 0.2, 0.4)

# We will now create a nice looking mesh by wrapping the edges in tubes,
# and putting fat spheres at the points.
extract = Vtk::ExtractEdges.new
extract.SetInputConnection(delny.GetOutputPort)
tubes = Vtk::TubeFilter.new
tubes.SetInputConnection(extract.GetOutputPort)
tubes.SetRadius(0.01)
tubes.SetNumberOfSides(6)
mapEdges = Vtk::PolyDataMapper.new
mapEdges.SetInputConnection(tubes.GetOutputPort)
edgeActor = Vtk::Actor.new
edgeActor.SetMapper(mapEdges)
edgeActor.GetProperty.SetColor(Vtk::Colors::Peacock)
edgeActor.GetProperty.SetSpecularColor(1, 1, 1)
edgeActor.GetProperty.SetSpecular(0.3)
edgeActor.GetProperty.SetSpecularPower(20)
edgeActor.GetProperty.SetAmbient(0.2)
edgeActor.GetProperty.SetDiffuse(0.8)

ball = Vtk::SphereSource.new
ball.SetRadius(0.025)
ball.SetThetaResolution(12)
ball.SetPhiResolution(12)
balls = Vtk::Glyph3D.new
balls.SetInputConnection(delny.GetOutputPort)
balls.SetSourceConnection(ball.GetOutputPort)
mapBalls = Vtk::PolyDataMapper.new
mapBalls.SetInputConnection(balls.GetOutputPort)
ballActor = Vtk::Actor.new
ballActor.SetMapper(mapBalls)
ballActor.GetProperty.SetColor(Vtk::Colors::Hot_pink)
ballActor.GetProperty.SetSpecularColor(1, 1, 1)
ballActor.GetProperty.SetSpecular(0.3)
ballActor.GetProperty.SetSpecularPower(20)
ballActor.GetProperty.SetAmbient(0.2)
ballActor.GetProperty.SetDiffuse(0.8)

# Create the rendering window, renderer, and interactive renderer
ren = Vtk::Renderer.new
renWin = Vtk::RenderWindow.new
renWin.AddRenderer(ren)
iren = Vtk::RenderWindowInteractor.new
iren.SetRenderWindow(renWin)

# Add the actors to the renderer, set the background and size
ren.AddActor(ballActor)
ren.AddActor(edgeActor)
ren.SetBackground(1, 1, 1)
renWin.SetSize(150, 150)

ren.ResetCamera
ren.GetActiveCamera.Zoom(1.5)

# Interact with the data.
iren.Initialize
renWin.Render
iren.Start
