#!/usr/bin/env ruby

# This example demonstrates how to use a programmable filter && how
# to use the special vtkDataSetToDataSet::GetOutput methods

require 'vtk'

include Math

# We create a 100 by 100 point plane to sample
plane = Vtk::PlaneSource.new
plane.SetXResolution(100)
plane.SetYResolution(100)

# We transform the plane by a factor of 10 on X && Y
transform = Vtk::Transform.new
transform.Scale(10, 10, 1)
transF = Vtk::TransformPolyDataFilter.new
transF.SetInputConnection(plane.GetOutputPort)
transF.SetTransform(transform)

# Compute Bessel function && derivatives. We'll use a programmable filter
# for this. Note the unusual GetInput & GetOutput methods.
besselF = Vtk::ProgrammableFilter.new
besselF.SetInputConnection(transF.GetOutputPort)

# The SetExecuteMethod takes a Python function as an argument
# In here is where all the processing is done.
bessel = Proc.new{
    input = besselF.GetPolyDataInput
    numPts = input.GetNumberOfPoints
    newPts = Vtk::Points.new
    derivs = Vtk::FloatArray.new

    for i in 0...numPts
        x = input.GetPoint(i)
        x0, x1 = x[0..1]

        r = sqrt(x0*x0+x1*x1)
        x2 = exp(-r)*cos(10.0*r)
        deriv = -exp(-r)*(cos(10.0*r)+10.0*sin(10.0*r))

        newPts.InsertPoint(i, x0, x1, x2)
        derivs.InsertValue(i, deriv) 
    end

    besselF.GetPolyDataOutput.CopyStructure(input)
    besselF.GetPolyDataOutput.SetPoints(newPts)
    besselF.GetPolyDataOutput.GetPointData.SetScalars(derivs)
}

besselF.SetExecuteMethod(bessel) 

# We warp the plane based on the scalar values calculated above
warp = Vtk::WarpScalar.new
warp.SetInput(besselF.GetPolyDataOutput)
warp.XYPlaneOn
warp.SetScaleFactor(0.5)


# We create a mapper && actor as usual. In the case we adjust the
# scalar range of the mapper to match that of the computed scalars
mapper = Vtk::PolyDataMapper.new
mapper.SetInput(warp.GetPolyDataOutput)
mapper.SetScalarRange(besselF.GetPolyDataOutput.GetScalarRange)
carpet = Vtk::Actor.new
carpet.SetMapper(mapper)

# Create the RenderWindow, Renderer
ren = Vtk::Renderer.new
renWin = Vtk::RenderWindow.new
renWin.AddRenderer(ren)
iren = Vtk::RenderWindowInteractor.new
iren.SetRenderWindow(renWin)

ren.AddActor(carpet)
renWin.SetSize(500, 500)

ren.ResetCamera
ren.GetActiveCamera.Zoom(1.5)

iren.Initialize
renWin.Render
iren.Start
