#!/usr/bin/env ruby

# This example demonstrates how to use boolean combinations of implicit
# functions to create a model of an ice cream cone.

require 'vtk'
require 'vtk/util'

# Create implicit function primitives. These have been carefully
# placed to give the effect that we want. We are going to use various
# combinations of these functions to create the shape we want; for
# example, we use planes intersected with a cone (which is infinite in
# extent) to get a finite cone.
cone = Vtk::Cone.new
cone.SetAngle(20)
vertPlane = Vtk::Plane.new
vertPlane.SetOrigin(0.1, 0, 0)
vertPlane.SetNormal(-1, 0, 0)
basePlane = Vtk::Plane.new
basePlane.SetOrigin(1.2, 0, 0)
basePlane.SetNormal(1, 0, 0)
iceCream = Vtk::Sphere.new
iceCream.SetCenter(1.333, 0, 0)
iceCream.SetRadius(0.5)
bite = Vtk::Sphere.new
bite.SetCenter(1.5, 0, 0.5)
bite.SetRadius(0.25)

# Combine primitives to build ice-cream cone. Clip the cone with planes.
theCone = Vtk::ImplicitBoolean.new
theCone.SetOperationTypeToIntersection
theCone.AddFunction(cone)
theCone.AddFunction(vertPlane)
theCone.AddFunction(basePlane)

# Take a bite out of the ice cream.
theCream = Vtk::ImplicitBoolean.new
theCream.SetOperationTypeToDifference
theCream.AddFunction(iceCream)
theCream.AddFunction(bite)

# The sample function generates a distance function from the implicit
# function (which in this case is the cone). This is then contoured to
# get a polygonal surface.
theConeSample = Vtk::SampleFunction.new
theConeSample.SetImplicitFunction(theCone)
theConeSample.SetModelBounds(-1, 1.5, -1.25, 1.25, -1.25, 1.25)
theConeSample.SetSampleDimensions(60, 60, 60)
theConeSample.ComputeNormalsOff
theConeSurface = Vtk::ContourFilter.new
theConeSurface.SetInputConnection(theConeSample.GetOutputPort)
theConeSurface.SetValue(0, 0.0)
coneMapper = Vtk::PolyDataMapper.new
coneMapper.SetInputConnection(theConeSurface.GetOutputPort)
coneMapper.ScalarVisibilityOff
coneActor = Vtk::Actor.new
coneActor.SetMapper(coneMapper)
coneActor.GetProperty.SetColor(Vtk::Colors::Chocolate)

# The same here for the ice cream.
theCreamSample = Vtk::SampleFunction.new
theCreamSample.SetImplicitFunction(theCream)
theCreamSample.SetModelBounds(0, 2.5, -1.25, 1.25, -1.25, 1.25)
theCreamSample.SetSampleDimensions(60, 60, 60)
theCreamSample.ComputeNormalsOff
theCreamSurface = Vtk::ContourFilter.new
theCreamSurface.SetInputConnection(theCreamSample.GetOutputPort)
theCreamSurface.SetValue(0, 0.0)
creamMapper = Vtk::PolyDataMapper.new
creamMapper.SetInputConnection(theCreamSurface.GetOutputPort)
creamMapper.ScalarVisibilityOff
creamActor = Vtk::Actor.new
creamActor.SetMapper(creamMapper)
creamActor.GetProperty.SetColor(Vtk::Colors::Mint)

# Create the usual rendering stuff
ren = Vtk::Renderer.new
renWin = Vtk::RenderWindow.new
renWin.AddRenderer(ren)
iren = Vtk::RenderWindowInteractor.new
iren.SetRenderWindow(renWin)

# Add the actors to the renderer, set the background and size
ren.AddActor(coneActor)
ren.AddActor(creamActor)
ren.SetBackground(1, 1, 1)
renWin.SetSize(250, 250)
ren.ResetCamera
ren.GetActiveCamera.Roll(90)
ren.GetActiveCamera.Dolly(1.5)
ren.ResetCameraClippingRange

iren.Initialize
renWin.Render
iren.Start
