#!/usr/bin/env ruby

# This example shows how to construct a surface from a point cloud.
# First we generate a volume using the
# vtkSurfaceReconstructionFilter. The volume values are a distance
# field. Once this is generated, the volume is countoured at a
# distance value of 0.0.

require 'vtk'
require 'vtk/util'
VTK_DATA_ROOT = vtkGetDataRoot

# Read some points. Use a programmable filter to read them.
pointSource = Vtk::ProgrammableSource.new

readPoints = Proc.new{
    output = pointSource.GetPolyDataOutput
    points = Vtk::Points.new
    output.SetPoints(points)

    file = File.open(File.join(VTK_DATA_ROOT, "Data/cactus.3337.pts"))

    line = file.gets
    while line
        data = line.split
        if data && data[0] == 'p'
            x, y, z = data[1].to_f, data[2].to_f, data[3].to_f
            points.InsertNextPoint(x, y, z)
        end
        line = file.gets
    end
}

pointSource.SetExecuteMethod(readPoints)


# Construct the surface && create isosurface.
surf = Vtk::SurfaceReconstructionFilter.new
surf.SetInput(pointSource.GetPolyDataOutput)

cf = Vtk::ContourFilter.new
cf.SetInputConnection(surf.GetOutputPort)
cf.SetValue(0, 0.0)

# Sometimes the contouring algorithm can create a volume whose gradient
# vector && ordering of polygon (using the right hand rule) are
# inconsistent. vtkReverseSense cures this problem.
reverse = Vtk::ReverseSense.new
reverse.SetInputConnection(cf.GetOutputPort)
reverse.ReverseCellsOn
reverse.ReverseNormalsOn

map = Vtk::PolyDataMapper.new
map.SetInputConnection(reverse.GetOutputPort)
map.ScalarVisibilityOff

surfaceActor = Vtk::Actor.new
surfaceActor.SetMapper(map)
surfaceActor.GetProperty.SetDiffuseColor(1.0000, 0.3882, 0.2784)
surfaceActor.GetProperty.SetSpecularColor(1, 1, 1)
surfaceActor.GetProperty.SetSpecular(0.4)
surfaceActor.GetProperty.SetSpecularPower(50)

# Create the RenderWindow, Renderer && both Actors
ren = Vtk::Renderer.new
renWin = Vtk::RenderWindow.new
renWin.AddRenderer(ren)
iren = Vtk::RenderWindowInteractor.new
iren.SetRenderWindow(renWin)

# Add the actors to the renderer, set the background && size
ren.AddActor(surfaceActor)
ren.SetBackground(1, 1, 1)
renWin.SetSize(400, 400)
ren.GetActiveCamera.SetFocalPoint(0, 0, 0)
ren.GetActiveCamera.SetPosition(1, 0, 0)
ren.GetActiveCamera.SetViewUp(0, 0, 1)
ren.ResetCamera
ren.GetActiveCamera.Azimuth(20)
ren.GetActiveCamera.Elevation(30)
ren.GetActiveCamera.Dolly(1.2)
ren.ResetCameraClippingRange

iren.Initialize
renWin.Render
iren.Start
