#!/usr/bin/env ruby

# This example demonstrates the use of vtkCardinalSpline.
# It creates random points and connects them with a spline

require 'vtk'
require 'vtk/util'

# This will be used later to get random numbers.
math = Vtk::Math.new

# Total number of points.
numberOfInputPoints = 10

# One spline for each direction.
aSplineX = Vtk::CardinalSpline.new
aSplineY = Vtk::CardinalSpline.new
aSplineZ = Vtk::CardinalSpline.new

# Generate random (pivot) points and add the corresponding
# coordinates to the splines.
# aSplineX will interpolate the x values of the points
# aSplineY will interpolate the y values of the points
# aSplineZ will interpolate the z values of the points
inputPoints = Vtk::Points.new
for i in 0...numberOfInputPoints
  x = math.Random(0, 1)
  y = math.Random(0, 1)
  z = math.Random(0, 1)
  aSplineX.AddPoint(i, x)
  aSplineY.AddPoint(i, y)
  aSplineZ.AddPoint(i, z)
  inputPoints.InsertPoint(i, x, y, z)
end
 

# The following section will create glyphs for the pivot points
# in order to make the effect of the spline more clear.

# Create a polydata to be glyphed.
inputData = Vtk::PolyData.new
inputData.SetPoints(inputPoints)

# Use sphere as glyph source.
balls = Vtk::SphereSource.new
balls.SetRadius(0.01)
balls.SetPhiResolution(10)
balls.SetThetaResolution(10)

glyphPoints = Vtk::Glyph3D.new
glyphPoints.SetInput(inputData)
glyphPoints.SetSource(balls.GetOutput)

glyphMapper = Vtk::PolyDataMapper.new
glyphMapper.SetInputConnection(glyphPoints.GetOutputPort)

glyph = Vtk::Actor.new
glyph.SetMapper(glyphMapper)
glyph.GetProperty.SetDiffuseColor(Vtk::Colors::Tomato)
glyph.GetProperty.SetSpecular(0.3)
glyph.GetProperty.SetSpecularPower(30)

# Generate the polyline for the spline.
points = Vtk::Points.new
profileData = Vtk::PolyData.new

# Number of points on the spline
numberOfOutputPoints = 400

# Interpolate x, y and z by using the three spline filters and
# create new points
for i in 0...numberOfOutputPoints
  t = (numberOfInputPoints-1.0)/(numberOfOutputPoints-1.0)*i
  points.InsertPoint(i, aSplineX.Evaluate(t), aSplineY.Evaluate(t),
                     aSplineZ.Evaluate(t))
end
 

# Create the polyline.
lines = Vtk::CellArray.new
lines.InsertNextCell(numberOfOutputPoints)
for i in 0...numberOfOutputPoints
  lines.InsertCellPoint(i)
end
 
profileData.SetPoints(points)
profileData.SetLines(lines)

# Add thickness to the resulting line.
profileTubes = Vtk::TubeFilter.new
profileTubes.SetNumberOfSides(8)
profileTubes.SetInput(profileData)
profileTubes.SetRadius(0.005)

profileMapper = Vtk::PolyDataMapper.new
profileMapper.SetInputConnection(profileTubes.GetOutputPort)

profile = Vtk::Actor.new
profile.SetMapper(profileMapper)
profile.GetProperty.SetDiffuseColor(Vtk::Colors::Banana)
profile.GetProperty.SetSpecular(0.3)
profile.GetProperty.SetSpecularPower(30)

# Now create the RenderWindow, Renderer and Interactor
ren = Vtk::Renderer.new
renWin = Vtk::RenderWindow.new
renWin.AddRenderer(ren)

iren = Vtk::RenderWindowInteractor.new
iren.SetRenderWindow(renWin)

# Add the actors
ren.AddActor(glyph)
ren.AddActor(profile)

renWin.SetSize(500, 500)

iren.Initialize
renWin.Render
iren.Start
