#!/usr/bin/env ruby

# This example demonstrates the use of the contour filter, and the use of
# the vtkSampleFunction to generate a volume of data samples from an
# implicit function.

require 'vtk'

# VTK supports implicit functions of the form f(x,y,z)=constant. These 
# functions can represent things spheres, cones, etc. Here we use a 
# general form for a quadric to create an elliptical data field.
quadric = Vtk::Quadric.new
quadric.SetCoefficients(0.5, 1, 0.2, 0, 0.1, 0, 0, 0.2, 0, 0)

# vtkSampleFunction samples an implicit function over the x-y-z range
# specified (here it defaults to -1,1 in the x,y,z directions).
sample = Vtk::SampleFunction.new
sample.SetSampleDimensions(30, 30, 30)
sample.SetImplicitFunction(quadric)

# Create five surfaces F(x,y,z) = constant between range specified. The
# GenerateValues() method creates n isocontour values between the range
# specified.
contours = Vtk::ContourFilter.new
contours.SetInputConnection(sample.GetOutputPort)
contours.GenerateValues(5, 0.0, 1.2)

contMapper = Vtk::PolyDataMapper.new
contMapper.SetInputConnection(contours.GetOutputPort)
contMapper.SetScalarRange(0.0, 1.2)

contActor = Vtk::Actor.new
contActor.SetMapper(contMapper)

# We'll put a simple outline around the data.
outline = Vtk::OutlineFilter.new
outline.SetInputConnection(sample.GetOutputPort)

outlineMapper = Vtk::PolyDataMapper.new
outlineMapper.SetInputConnection(outline.GetOutputPort)

outlineActor = Vtk::Actor.new
outlineActor.SetMapper(outlineMapper)
outlineActor.GetProperty.SetColor(0,0,0)

# The usual rendering stuff.
ren = Vtk::Renderer.new
renWin = Vtk::RenderWindow.new
renWin.AddRenderer(ren)
iren = Vtk::RenderWindowInteractor.new
iren.SetRenderWindow(renWin)

ren.SetBackground(1, 1, 1)
ren.AddActor(contActor)
ren.AddActor(outlineActor)

iren.Initialize
renWin.Render
iren.Start
