#!/usr/bin/env ruby

# This example demonstrates how to extract "computational planes" from
# a structured dataset. Structured data has a natural, logical
# coordinate system based on i-j-k indices. Specifying imin,imax,
# jmin,jmax, kmin,kmax pairs can indicate a point, line, plane, or
# volume of data.
#
# In this example, we extract three planes and warp them using scalar
# values in the direction of the local normal at each point. This
# gives a sort of "velocity profile" that indicates the nature of the
# flow.

require 'vtk'
require 'vtk/util'
VTK_DATA_ROOT = vtkGetDataRoot

# Here we read data from a annular combustor. A combustor burns fuel
# and air in a gas turbine (e.g., a jet engine) and the hot gas
# eventually makes its way to the turbine section.
pl3d = Vtk::PLOT3DReader.new
pl3d.SetXYZFileName(VTK_DATA_ROOT + "/Data/combxyz.bin")
pl3d.SetQFileName(VTK_DATA_ROOT + "/Data/combq.bin")
pl3d.SetScalarFunctionNumber(100)
pl3d.SetVectorFunctionNumber(202)
pl3d.Update

# Planes are specified using a imin,imax, jmin,jmax, kmin,kmax
# coordinate specification. Min and max i,j,k values are clamped to 0
# and maximum value.
plane = Vtk::StructuredGridGeometryFilter.new
plane.SetInputConnection(pl3d.GetOutputPort)
plane.SetExtent(10, 10, 1, 100, 1, 100)
plane2 = Vtk::StructuredGridGeometryFilter.new
plane2.SetInputConnection(pl3d.GetOutputPort)
plane2.SetExtent(30, 30, 1, 100, 1, 100)
plane3 = Vtk::StructuredGridGeometryFilter.new
plane3.SetInputConnection(pl3d.GetOutputPort)
plane3.SetExtent(45, 45, 1, 100, 1, 100)

# We use an append filter because that way we can do the warping,
# etc. just using a single pipeline and actor.
appendF = Vtk::AppendPolyData.new
appendF.AddInput(plane.GetOutput)
appendF.AddInput(plane2.GetOutput)
appendF.AddInput(plane3.GetOutput)
warp = Vtk::WarpScalar.new
warp.SetInputConnection(appendF.GetOutputPort)
warp.UseNormalOn
warp.SetNormal(1.0, 0.0, 0.0)
warp.SetScaleFactor(2.5)
normals = Vtk::PolyDataNormals.new
normals.SetInput(warp.GetPolyDataOutput)
normals.SetFeatureAngle(60)
planeMapper = Vtk::PolyDataMapper.new
planeMapper.SetInputConnection(normals.GetOutputPort)
planeMapper.SetScalarRange(pl3d.GetOutput.GetScalarRange)
planeActor = Vtk::Actor.new
planeActor.SetMapper(planeMapper)

# The outline provides context for the data and the planes.
outline = Vtk::StructuredGridOutlineFilter.new
outline.SetInputConnection(pl3d.GetOutputPort)
outlineMapper = Vtk::PolyDataMapper.new
outlineMapper.SetInputConnection(outline.GetOutputPort)
outlineActor = Vtk::Actor.new
outlineActor.SetMapper(outlineMapper)
outlineActor.GetProperty.SetColor(0, 0, 0)

# Create the usual graphics stuff.
ren = Vtk::Renderer.new
renWin = Vtk::RenderWindow.new
renWin.AddRenderer(ren)
iren = Vtk::RenderWindowInteractor.new
iren.SetRenderWindow(renWin)

ren.AddActor(outlineActor)
ren.AddActor(planeActor)
ren.SetBackground(1, 1, 1)
renWin.SetSize(500, 500)

# Create an initial view.
cam1 = ren.GetActiveCamera
cam1.SetClippingRange(3.95297, 50)
cam1.SetFocalPoint(8.88908, 0.595038, 29.3342)
cam1.SetPosition(-12.3332, 31.7479, 41.2387)
cam1.SetViewUp(0.060772, -0.319905, 0.945498)

iren.Initialize
renWin.Render
iren.Start
