#!/usr/bin/env ruby

# Perform psuedo volume rendering in a structured grid by compositing
# translucent cut planes. This same trick can be used for unstructured
# grids. Note that for better results, more planes can be created. Also,
# if your data is vtkImageData, there are much faster methods for volume
# rendering.

require 'vtk'
require 'vtk/util'

VTK_DATA_ROOT = vtkGetDataRoot

# Create pipeline. Read structured grid data.
pl3d = Vtk::PLOT3DReader.new
pl3d.SetXYZFileName(VTK_DATA_ROOT + "/Data/combxyz.bin")
pl3d.SetQFileName(VTK_DATA_ROOT + "/Data/combq.bin")
pl3d.SetScalarFunctionNumber(100)
pl3d.SetVectorFunctionNumber(202)
pl3d.Update

# A convenience, use this filter to limit data for experimentation.
extract = Vtk::ExtractGrid.new
extract.SetVOI(1, 55, -1000, 1000, -1000, 1000)
extract.SetInputConnection(pl3d.GetOutputPort)

# The (implicit) plane is used to do the cutting
@plane = Vtk::Plane.new
@plane.SetOrigin(0, 4, 2)
@plane.SetNormal(0, 1, 0)

# The cutter is set up to process each contour value over all cells
# (SetSortByToSortByCell). This results in an ordered output of polygons
# which is key to the compositing.
@cutter = Vtk::Cutter.new
@cutter.SetInputConnection(extract.GetOutputPort)
@cutter.SetCutFunction(@plane)
@cutter.GenerateCutScalarsOff
@cutter.SetSortByToSortByCell

@clut = Vtk::LookupTable.new
@clut.SetHueRange(0, 0.67)
@clut.Build

cutterMapper = Vtk::PolyDataMapper.new
cutterMapper.SetInputConnection(@cutter.GetOutputPort)
cutterMapper.SetScalarRange(0.18, 0.7)
cutterMapper.SetLookupTable(@clut)

cut = Vtk::Actor.new
cut.SetMapper(cutterMapper)

# Add in some surface geometry for interest.
iso = Vtk::ContourFilter.new
iso.SetInputConnection(pl3d.GetOutputPort)
iso.SetValue(0, 0.22)
normals = Vtk::PolyDataNormals.new
normals.SetInputConnection(iso.GetOutputPort)
normals.SetFeatureAngle(45)
isoMapper = Vtk::PolyDataMapper.new
isoMapper.SetInputConnection(normals.GetOutputPort)
isoMapper.ScalarVisibilityOff
isoActor = Vtk::Actor.new
isoActor.SetMapper(isoMapper)
isoActor.GetProperty.SetDiffuseColor(Vtk::Colors::Tomato)
isoActor.GetProperty.SetSpecularColor(Vtk::Colors::White)
isoActor.GetProperty.SetDiffuse(0.8)
isoActor.GetProperty.SetSpecular(0.5)
isoActor.GetProperty.SetSpecularPower(30)

outline = Vtk::StructuredGridOutlineFilter.new
outline.SetInputConnection(pl3d.GetOutputPort)
outlineTubes = Vtk::TubeFilter.new
outlineTubes.SetInputConnection(outline.GetOutputPort)
outlineTubes.SetRadius(0.1)

outlineMapper = Vtk::PolyDataMapper.new
outlineMapper.SetInputConnection(outlineTubes.GetOutputPort)
outlineActor = Vtk::Actor.new
outlineActor.SetMapper(outlineMapper)

# Create the RenderWindow, Renderer and Interactor
ren = Vtk::Renderer.new
@renWin = Vtk::RenderWindow.new
@renWin.AddRenderer(ren)
iren = Vtk::RenderWindowInteractor.new
iren.SetRenderWindow(@renWin)

# Add the actors to the renderer, set the background and size
ren.AddActor(outlineActor)
outlineActor.GetProperty.SetColor(Vtk::Colors::Banana)
ren.AddActor(isoActor)
isoActor.VisibilityOn
ren.AddActor(cut)
@opacity = 0.1
cut.GetProperty.SetOpacity(1)
ren.SetBackground(1, 1, 1)
@renWin.SetSize(640, 480)

@cam1 = ren.GetActiveCamera
@cam1.SetClippingRange(3.95297, 50)
@cam1.SetFocalPoint(9.71821, 0.458166, 29.3999)
@cam1.SetPosition(2.7439, -37.3196, 38.7167)
@cam1.ComputeViewPlaneNormal
@cam1.SetViewUp(-0.16123, 0.264271, 0.950876)

# Cut: generates n cut planes normal to camera's view plane
def Cut(n)
  @plane.SetNormal(@cam1.GetViewPlaneNormal)
  @plane.SetOrigin(@cam1.GetFocalPoint)
  @cutter.GenerateValues(n, -5, 5)
  @clut.SetAlphaRange(@opacity, @opacity)
  @renWin.Render
end
 

# Generate 10 cut planes
Cut(20)

iren.Initialize
@renWin.Render
iren.Start
